from typing import Any

from tfy_assistant_framework.swarm import Agent, client


class Chat(Agent[Any]): ...


chat_agent = Chat(
    instructions="Chat with the user",
)


async def run_chat_assistant() -> None:
    async for agent_run in client.run(agent=chat_agent, messages=[], interactive=True):
        agent_run.debug_log()


async def main() -> None:
    await run_chat_assistant()


if __name__ == "__main__":
    import asyncio

    asyncio.run(main())
